/*
 * stm_core.c
 *
 *  Created on: 11. 2. 2021
 *      Author: tomas
 */

#include "stm_core.h"


bool SetClock100MHz(eClockSources clkSrc)
{
  uint32_t t;

  if(clkSrc == clockSourceHSE)
  {
    if(!(RCC->CR & RCC_CR_HSEON))
    {
      RCC->CR |= RCC_CR_HSEON;

      t = 200;
      while(!(RCC->CR & RCC_CR_HSEON) && t)
        t--;
      if(!t)
        return false;
    }
  }

if(!(RCC->CR & RCC_CR_HSION))
{
  RCC->CR |= RCC_CR_HSION;

  t = 100;
  while(!(RCC->CR & RCC_CR_HSION) && t)
    t--;
  if(!t)
    return false;
}

if(RCC->CR & RCC_CR_PLLON)
{
  RCC->CR &= ~RCC_CR_PLLON;
}

RCC->CFGR &= ~(RCC_CFGR_SW);

RCC->CFGR = 0;

RCC->CFGR |= 0
    | 0 << 13
    | 4 << 10
    | 0 << 4
    ;

if(clkSrc == clockSourceHSI)
{
  RCC->PLLCFGR &= ~(1 << 22);

  RCC->PLLCFGR &= ~RCC_PLLCFGR_PLLM;
  RCC->PLLCFGR |= 8 << 0;
  RCC->PLLCFGR &= ~RCC_PLLCFGR_PLLN;
  RCC->PLLCFGR |= 100 << 6;
  RCC->PLLCFGR &= ~RCC_PLLCFGR_PLLP;
  RCC->PLLCFGR |= 0 << 16;

}

if(clkSrc == clockSourceHSE)
{
  RCC->PLLCFGR &= ~(1 << 22);
  RCC->PLLCFGR |= RCC_PLLCFGR_PLLSRC_HSE;

  RCC->PLLCFGR &= ~RCC_PLLCFGR_PLLM;
  RCC->PLLCFGR |= 4 << 0;
  RCC->PLLCFGR &= ~RCC_PLLCFGR_PLLN;
  RCC->PLLCFGR |= 100 << 6;
  RCC->PLLCFGR &= ~RCC_PLLCFGR_PLLP;
  RCC->PLLCFGR |= 0 << 16;
}

RCC->CR |= RCC_CR_PLLON;

t = 100;
while(!(RCC->CR & RCC_CR_PLLON) && t)
  t--;
if(!t)
  return false;

FLASH->ACR &= ~(0x0f << 0);
FLASH->ACR |= (3<<0);

PWR->CR |= PWR_CR_VOS_0 | PWR_CR_VOS_1;

RCC->CFGR |= RCC_CFGR_SW_PLL;
t = 100;
while(!((RCC->CFGR & 0x0c) == RCC_CFGR_SWS_PLL) && t)
  t--;
if(!t)
  return false;

  return true;
}

bool SetClockHSI(void)
{
  uint32_t t;

  if(!(RCC->CR & RCC_CR_HSION))
  {
    RCC->CR |= RCC_CR_HSION;

    t=100;
    while(!(RCC->CR & RCC_CR_HSION) && t)
      t--;
    if(!t)
      return false;
  }

  RCC->CFGR &= ~RCC_CFGR_SW;
  t=100;
  while(!((RCC->CR & 0x0c) == RCC_CFGR_SWS_HSI )&& t)
        t--;
      if(!t)
        return false;

      return true;
 }




uint32_t GetBusClock(eBusClocks clk)
{
  uint32_t bitval = 0;
  uint32_t divider = 1;

#if defined(STM32F411xE)
  switch(clk)
  {
    case busClockAHB:
      bitval = (RCC->CFGR & (0x0f  << 4)) >> 4;
      if (bitval & 0x8)
        divider = 1 << ((bitval & 0x07) + 1);
      else
        divider = 1;
      break;
    case busClockAPB1:
    case timersClockAPB1:
      bitval = (RCC->CFGR & (0x07  << 10)) >> 10;
      if (bitval & 0x4)
        divider = 1 << ((bitval & 0x03) + 1);
      else
        divider = 1;
      break;
    case busClockAPB2:
    case timersClockAPB2:
      bitval = (RCC->CFGR >> 13) & 0x07;
      if (bitval & 0x4)
        divider = 1 << ((bitval & 0x03) + 1);
      else
        divider = 1;
      break;
    default:
      return 0;
  }

SystemCoreClockUpdate();

if(((clk == timersClockAPB1) || (clk == timersClockAPB1)) && (divider > 1))
  return SystemCoreClock / divider * 2;
else
  return SystemCoreClock / divider;
#else
#error Valid controller not set
#endif

}

uint32_t GetTimerClock(int timerNum)
{
uint32_t timerClock = 0;

#if defined(STM32F411xE)
  switch(timerNum)
  {
    case 1:
    case 9:
    case 10:
    case 11:
      timerClock = GetBusClock(timersClockAPB2);
      break;
    case 2:
    case 3:
    case 4:
    case 5:
      timerClock = GetBusClock(timersClockAPB1);
      break;
  }
#endif

  return timerClock;
}

bool Nucleo_SetPinGPIO(GPIO_TypeDef *gpio, uint32_t bitnum, eIoPortModes mode)
{
  uint32_t enr_mask = 0;    //hodnota do xxENR registru
  uint32_t rstr_mask = 0;   //hodnota do xxRSTR registru
  //detekce kterz GPIO
  switch ((uint32_t)gpio)
  {
    case (uint32_t)GPIOA:
        enr_mask = RCC_AHB1ENR_GPIOAEN;
        rstr_mask = RCC_AHB1RSTR_GPIOARST;
      break;
    case (uint32_t)GPIOB:
        enr_mask = RCC_AHB1ENR_GPIOBEN;
        rstr_mask = RCC_AHB1RSTR_GPIOBRST;
      break;
    case (uint32_t)GPIOC:
        enr_mask = RCC_AHB1ENR_GPIOCEN;
        rstr_mask = RCC_AHB1RSTR_GPIOCRST;
      break;
    case (uint32_t)GPIOD:
        enr_mask = RCC_AHB1ENR_GPIODEN;
        rstr_mask = RCC_AHB1RSTR_GPIODRST;
      break;
    case (uint32_t)GPIOE:
        enr_mask = RCC_AHB1ENR_GPIOEEN;
        rstr_mask = RCC_AHB1RSTR_GPIOERST;
      break;
#if defined(STM32F411xE)      // 411RE nema GPIOF a GPIOG
#else
    case (uint32_t)GPIOF:
        enr_mask = RCC_AHB1ENR_GPIOFEN;
        rstr_mask = RCC_AHB1RSTR_GPIOFRST;
      break;
    case (uint32_t)GPIOG:
        enr_mask = RCC_AHB1ENR_GPIOGEN;
        rstr_mask = RCC_AHB1RSTR_GPIOGRST;
      break;
#endif
    case (uint32_t)GPIOH:
        enr_mask = RCC_AHB1ENR_GPIOHEN;
        rstr_mask = RCC_AHB1RSTR_GPIOHRST;
      break;
  }

  if ((enr_mask == 0)||(rstr_mask == 0))
    return false;     //priznak chyby

  //inicializace vybraneho
  if (!(RCC->AHB1ENR & enr_mask))
  {
       RCC->AHB1ENR |= enr_mask;
       RCC->AHB1RSTR |= rstr_mask;
       RCC->AHB1RSTR &= ~rstr_mask;
  }

  //nastaveni konfiguracnich bitu do def stavu (nemusi byt od drive)
  gpio->MODER &= ~(0x03<<(2 * bitnum));   //vynuluj prislusne 2 bity v registru
  gpio->PUPDR &= ~(0x03<<(2 * bitnum));   //vynuluj prislusne 2 bity v registru
  gpio->OSPEEDR &= ~(0x03<<(2 * bitnum));   //vynuluj prislusne 2 bity v registru

  //nastaveni registru podle typu vystupu/vstupu
  switch(mode)
  {
    case ioPortOutputOC:
    case ioPortOutputPP:
      gpio->MODER |= 0x01 << (2 * bitnum);    // 01 - output
      gpio->OSPEEDR |= 0x03 << (2 * bitnum);  // 11 - high speed
      gpio->PUPDR &= ~(0x03<<(2 * bitnum));   // 00 - no PU/PD

      if(mode == ioPortOutputOC)              // open colector?
        gpio->OTYPER |= 0x01 << bitnum;       // 1 - OC
      else
        gpio->OTYPER &= ~(0x01 << bitnum);    // 0 -pushpull
      break;

    case ioPortInputPU: // moder bits 00 = input
      gpio->PUPDR &= ~(0x03 << (2 * bitnum)); // clear bits
      gpio->PUPDR |= 0x01 << (2 * bitnum); // 01 = pull-up
      break;
    case ioPortInputPD:
      gpio->PUPDR &= ~(0x03 << (2 * bitnum)); // clear bits
      gpio->PUPDR |= 0x02 << (2 * bitnum); // 10 = pull-down
      break;
    case ioPortInputFloat: // 00 = input mode, nothing to do
      gpio->PUPDR &= ~(0x03 << (2 * bitnum)); // 00 = no pull-up/dn
      break;
    case ioPortAnalog: // 11 -analog mode
      gpio->MODER |= 0x03 << (2 * bitnum); // set bits
      break;
    case ioPortAlternatePP:
    case ioPortAlternateOC:
      gpio->MODER |= 0x00000002 << (2 * bitnum);    // set bits

      if(mode == ioPortAlternateOC)
        gpio->OTYPER |= 0x00000001 << bitnum;       // 1 - OC
      else
        gpio->OTYPER &= ~(0x00000001 << bitnum);    // 0 -pushpull

      gpio->OSPEEDR |= 0x00000003 << (2 * bitnum);  // 11 - high speed
      gpio->PUPDR &= ~(0x00000003<<(2 * bitnum));   // 00 - no PU/PD

      break;
    default: // neznamy rezim ?
      return false; // priznak chyby
  }

  return true;
}

bool Nucleo_SetAFGPIO(GPIO_TypeDef *gpio, uint32_t bitnum, uint32_t afValue)
{
  gpio->AFR[(bitnum < 8) ? 0 : 1] &= ~(0x0f << (4 * (bitnum & 0x07)));   //vynuluj AF bity
  gpio->AFR[(bitnum < 8) ? 0 : 1] |= ((afValue & 0x0f) << (4 * (bitnum & 0x07)));   //nastav AF bity
  return true;
}

void GPIOToggle(GPIO_TypeDef *gpio, uint32_t bitnum)
{
  gpio->ODR ^= 1 << bitnum;
}

bool GPIORead(GPIO_TypeDef *gpio, uint32_t bitnum)
{
  return ((gpio->IDR & (1 << bitnum))!= 0);     //porovnani = vynucena logicka hodnota
}

void GPIOWrite(GPIO_TypeDef *gpio, uint32_t bitnum, bool state)
{
  gpio->BSRR = (state) ? (1<<(bitnum)) : ((1 <<(bitnum))<<16);      //lower 16bit - set to 1, higher 16bit - reset to 0
}
